using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net.Mail;
using System.Security.Cryptography.X509Certificates;
using Microsoft.Azure.Cis.Activities.Contracts;
using Microsoft.Azure.Cis.Activities.Contracts.Events;
using Microsoft.Azure.Cis.Activities.Contracts.Planner;

/*
 * This class could be used when you want to 
 */
namespace FleetAGC.Activities.Util
{
    public class ContextConversion
    {
        static public IActivityContext plan2activity<T>(T obj) where T : IPlanContext
        {
            return new PlanContext2ActivityContext<T>(ref obj);
        }
    }
    
    internal class __dict_isettings : Dictionary<string, string>, ISettings
    {
        public IDictionary<string, string> GetAllSettings()
        {
            return this;
        }
        public bool IsTrue(string settingName)
        {
            return ContainsKey(settingName);
        }
        public ICollection<string> Keys => base.Keys;
        public ICollection<string> Values => base.Values;
    }
    
    /*
     * This is std::forward<T> in C++, to make silly csc happy about missing existing members in PlanContext2ActivityContext.
     * I'm very unhappy that C# has NO meta-programming at all.
     *
     * This forwarder class has included all stateful data, because this IS NOT an actual type function, this IS a data function. 
     */
    internal class __fake_cxx_std_forward<T> : IPlanContext where T : IPlanContext
    {
        public T realbase;
        public __fake_cxx_std_forward(ref T _realbase)
        {
            realbase = _realbase;
            GlobalSettings = realbase.GlobalSettings;
            SystemSettings = realbase.SystemSettings;
            Tenant = realbase.Tenant;
        }

        public ISettings GlobalSettings { get; }
        public IReadOnlySettings SystemSettings { get; }
        public void TraceStart(string format, params object[] args)
        {
            realbase.TraceStart(format, args);
        }

        public void TraceStart(string message)
        {
            realbase.TraceStart(message);
        }

        public void TraceStop(string format, params object[] args)
        {
            realbase.TraceStop(format, args);
        }

        public void TraceStop(string message)
        {
            realbase.TraceStop(message);
        }

        public void TraceVerbose(string format, params object[] args)
        {
            realbase.TraceVerbose(format, args);
        }

        public void TraceVerbose(string message)
        {
            realbase.TraceVerbose(message);
        }

        public void TraceInfo(string format, params object[] args)
        {
            realbase.TraceInfo(format, args);
        }

        public void TraceInfo(string message)
        {
            realbase.TraceInfo(message);
        }

        public void TraceWarning(string format, params object[] args)
        {
            realbase.TraceWarning(format, args);
        }

        public void TraceWarning(string message)
        {
            realbase.TraceWarning(message);
        }

        public void TraceError(string format, params object[] args)
        {
            realbase.TraceError(format, args);
        }

        public void TraceError(string message)
        {
            realbase.TraceError(message);
        }

        public TraceSource GetActiveTraceSource()
        {
            return realbase.GetActiveTraceSource();
        }

        public TraceListener GetActivityContextTraceListener()
        {
            return realbase.GetActivityContextTraceListener();
        }

        public string GetCredentialSetting(CredentialSettingType credentialSettingType, string securityId)
        {
            return realbase.GetCredentialSetting(credentialSettingType, securityId);
        }

        public X509Certificate2 GetManagedTenantCert()
        {
            return GetManagedTenantCert();
        }

        public X509Certificate2 GetProtectedSciTenantCert()
        {
            return GetProtectedSciTenantCert();
        }

        public string Tenant { get; }
    }
    
    /*
     * This class is completely auto-generated by Jetbrains Rider.
     * DO NOT modify by hand. 
     */
    internal class PlanContext2ActivityContext<T> : __fake_cxx_std_forward<T>, IActivityContext where T : IPlanContext
    {
        public PlanContext2ActivityContext(ref T realbase) : base(ref realbase)
        {
            // Manual patch: simulate some useful elements to empty, instead of null. 
            InputSettings = ActivityPersistedSettings = new __dict_isettings();
        }

        public bool IsSingleStep()
        {
            throw new NotImplementedException();
        }

        public string TaskDisplayName { get; }
        public string TaskId { get; }
        public string TaskLink { get; }
        public string TaskNotes { get; }
        public Guid RunId { get; }
        public string JobDisplayName { get; }
        public string JobId { get; }
        public string JobType { get; }
        public string Workflow { get; }
        public string ProjectRoot { get; }
        public string ScratchRoot { get; }
        public string ComponentName { get; }
        public void SignalActivityEvent(string eventName, string rootJobId)
        {
            throw new NotImplementedException();
        }

        public void SignalActivityEvent(string eventName, string rootJobId, Dictionary<string, string> eventArgs)
        {
            throw new NotImplementedException();
        }

        public void SignalActivityEvent(string eventName, string rootJobId, string eventArgs)
        {
            throw new NotImplementedException();
        }

        public void SignalActivityEvent(string eventName, string rootJobId, string senderId, Dictionary<string, string> eventArgs,
            Dictionary<string, string> eventScopes)
        {
            throw new NotImplementedException();
        }

        public void UpdateActivityEventArgs(string eventName, string rootJobId, string argName, string argValue, bool append)
        {
            throw new NotImplementedException();
        }

        public IActivityEventData GetActivityEvent(string eventName, string rootJobId)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<IActivityEventData> GetActivityEvents(string eventName)
        {
            throw new NotImplementedException();
        }

        public void RegisterActivityEvent(string eventName, string rootJobId)
        {
            throw new NotImplementedException();
        }

        public IEventData GetEventWithPrimaryScope(string eventName, KeyValuePair<string, string> scope)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<IEventData> GetEventsWithSecondaryScope(string eventName, IDictionary<string, string> scopes)
        {
            throw new NotImplementedException();
        }

        public void SignalEvent(IEventData eventData)
        {
            throw new NotImplementedException();
        }

        public void SignalEventForAllClouds(IEventData eventData)
        {
            throw new NotImplementedException();
        }

        public void RegisterEvent(IEventData eventData)
        {
            throw new NotImplementedException();
        }

        public void UpdateEvent(IEventData eventData)
        {
            throw new NotImplementedException();
        }

        public void ResetEvent(string eventName)
        {
            throw new NotImplementedException();
        }

        public void ResetEvent(string eventName, KeyValuePair<string, string> primaryScope)
        {
            throw new NotImplementedException();
        }

        public void CancelEvent(string eventName)
        {
            throw new NotImplementedException();
        }

        public void CancelEvent(string eventName, KeyValuePair<string, string> primaryScope)
        {
            throw new NotImplementedException();
        }

        public void ReplanWorkflow()
        {
            throw new NotImplementedException();
        }

        public void RescheduleActivity(TimeSpan delayDuration)
        {
            throw new NotImplementedException();
        }

        public void ScheduleChildActivity(Type childActivity, string displayName, Dictionary<string, string> inputSettings)
        {
            throw new NotImplementedException();
        }

        public void ScheduleChildActivity(Type childActivity, string displayName, Dictionary<string, string> inputSettings,
            RescheduleCallback callback)
        {
            throw new NotImplementedException();
        }

        public long FileIncident(string exceptionSourceName, string exceptionComponent, Exception unhandledException,
            string incidentFilingPolicy = "", string externalSystemType = "Tfs")
        {
            throw new NotImplementedException();
        }

        public long FileIncident(string exceptionSourceName, string exceptionComponent, Exception unhandledException, string routingId,
            string correlationId, string incidentFilingPolicy = "", string externalSystemType = "Tfs")
        {
            throw new NotImplementedException();
        }

        public long FileIncident(string activityName, string componentName, string providedIncidentTitle,
            string providedIncidentDetails, long incidentIdToReuse, bool incidentIsException = false,
            bool needToRestartWacb = false, string incidentFilingPolicy = "", string externalSystemType = "Tfs")
        {
            throw new NotImplementedException();
        }

        public long FileIncident(string activityName, string componentName, string providedIncidentTitle,
            string providedIncidentDetails, string routingId, string correlationId, long incidentIdToReuse,
            bool incidentIsException = false, bool needToRestartWacb = false, string incidentFilingPolicy = "",
            string externalSystemType = "Tfs")
        {
            throw new NotImplementedException();
        }

        public long FileIncident(string activityName, string componentName, string taskUniqueName, string providedIncidentTitle,
            string providedIncidentDetails, string routingId, string correlationId, long incidentIdToReuse,
            bool incidentIsException = false, bool needToRestartWacb = false, string incidentFilingPolicy = "",
            string externalSystemType = "Tfs")
        {
            throw new NotImplementedException();
        }

        public long FileIncident(string activityName, string componentName, string taskUniqueName, string providedIncidentTitle,
            string providedIncidentDetails, string routingId, string correlationId, long incidentIdToReuse,
            bool formatTitleAndDescription, bool incidentIsException = false, bool needToRestartWacb = false,
            string incidentFilingPolicy = "", string externalSystemType = "Tfs")
        {
            throw new NotImplementedException();
        }

        public long FileIncident(IDictionary<IncidentParameters, string> incidentParams = null, bool ignoreIncidentTitlePrefix = false,
            bool createWorkflowIncident = true, bool createOperationHistory = true)
        {
            throw new NotImplementedException();
        }

        public bool LinkIncident(string incidentId, bool createOperationHistory = true)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<string> GetIncidentIds()
        {
            throw new NotImplementedException();
        }

        public IEnumerable<string> GetAllIncidentIdsForActivity()
        {
            throw new NotImplementedException();
        }

        public IEnumerable<string> GetAllIncidentIdsForActivity(string externSystemTypes)
        {
            throw new NotImplementedException();
        }

        public string ScheduleRemoteJob(bool rescheduleActivity, RunRemoteJobParameters parameters)
        {
            throw new NotImplementedException();
        }

        public Dictionary<string, string> ReadRemoteJobSettings()
        {
            throw new NotImplementedException();
        }

        public RemoteJobStatus GetRemoteJobStatus()
        {
            throw new NotImplementedException();
        }

        public string GetRemoteJobId()
        {
            throw new NotImplementedException();
        }

        public string GetRootJobId()
        {
            throw new NotImplementedException();
        }

        public string GetJobSeverity()
        {
            throw new NotImplementedException();
        }

        public string CreateJob(CreateJobParameters job)
        {
            throw new NotImplementedException();
        }

        public JobInfo GetJob(string jobType, string jobId, string cloud = "")
        {
            throw new NotImplementedException();
        }

        public Dictionary<string, string> GetJobSettings(string jobType, string jobId, string cloud = "")
        {
            throw new NotImplementedException();
        }

        public void AbortJob(string jobType, string jobId, string cloud = "")
        {
            throw new NotImplementedException();
        }

        public void StartJob(string jobType, string jobId, string cloud = "")
        {
            throw new NotImplementedException();
        }

        public void RestartJob(string jobType, string jobId, string cloud = "")
        {
            throw new NotImplementedException();
        }

        public void UpdateJobSeverity(string jobType, string jobId, string jobSeverity)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<string> GetSubJobIds(string jobType, string jobId)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<TaskInfo> GetTasks(string jobType, string jobId)
        {
            throw new NotImplementedException();
        }

        public ISettings LoadNamespaceSettings(string namespaceName)
        {
            throw new NotImplementedException();
        }

        public void LogSettingSource(string name, string method)
        {
            throw new NotImplementedException();
        }

        public IReadOnlySettings InputSettings { get; }
        public ISettings ActivityPersistedSettings { get; }
        public IComponentSettings ComponentSettings { get; }
        public void UpdateProgress(string progressUpdate)
        {
            throw new NotImplementedException();
        }

        public void UpdateProgressLink(string progressReportLink)
        {
            throw new NotImplementedException();
        }

        public void UpdateProgressLink(Uri progressReportLink, string progressUpdate)
        {
            throw new NotImplementedException();
        }

        public void UpdateProgressLink(string progressReportLink, string progressUpdate)
        {
            throw new NotImplementedException();
        }

        public void UpdateProgressLinks(IEnumerable<Tuple<Uri, string>> links)
        {
            throw new NotImplementedException();
        }

        public void UpdateCustomState(string customState, string customStateReportLink = "")
        {
            throw new NotImplementedException();
        }

        public void UpdateWorkflowCustomState(string customState)
        {
            throw new NotImplementedException();
        }

        public void ReportOperation(string operation, string note, bool taskLevelOperation = false)
        {
            throw new NotImplementedException();
        }

        public void UpdateLastProgressReportTime(DateTime lastProgressReportTime)
        {
            throw new NotImplementedException();
        }

        public void UpdateBlockedReason(string message)
        {
            throw new NotImplementedException();
        }

        public string ProgressReportLink { get; }
        public IEnumerable<IActivityTicket> GetActivityTickets()
        {
            throw new NotImplementedException();
        }

        public void AddOrUpdateTicket(long ticketId, string ticketingSystem, ActivityTicketState ticketState)
        {
            throw new NotImplementedException();
        }

        public bool AcquireResources(IDictionary<string, int> resourcesNeeded, int waitingTimeoutInMinute)
        {
            throw new NotImplementedException();
        }

        public void ReleaseResources()
        {
            throw new NotImplementedException();
        }

        public void AddMetric(string name, int value)
        {
            throw new NotImplementedException();
        }

        public void SendEmail(string to, string subject, string body, bool isHtml = false, string ccRecipients = null,
            IEnumerable<string> attachmentFilePaths = null, MailPriority priority = MailPriority.Normal)
        {
            throw new NotImplementedException();
        }
    }

}
